/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.templateconcepts.call.resource.templatecall.mopp;

/**
 * The TemplatecallTokenResolverFactory class provides access to all generated
 * token resolvers. By giving the name of a defined token, the corresponding
 * resolve can be obtained. Despite the fact that this class is called
 * TokenResolverFactory is does NOT create new token resolvers whenever a client
 * calls methods to obtain a resolver. Rather, this class maintains a map of all
 * resolvers and creates each resolver at most once.
 */
public class TemplatecallTokenResolverFactory implements org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolverFactory {
	
	private java.util.Map<String, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver> tokenName2TokenResolver;
	private java.util.Map<String, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver> featureName2CollectInTokenResolver;
	private static org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver defaultResolver = new org.emftext.language.templateconcepts.call.resource.templatecall.analysis.TemplatecallDefaultTokenResolver();
	
	public TemplatecallTokenResolverFactory() {
		tokenName2TokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver>();
		featureName2CollectInTokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver>();
		registerTokenResolver("QUOTED_34_34", new org.emftext.language.templateconcepts.call.resource.templatecall.analysis.TemplatecallQUOTED_34_34TokenResolver());
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver createTokenResolver(String tokenName) {
		return internalCreateResolver(tokenName2TokenResolver, tokenName);
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver createCollectInTokenResolver(String featureName) {
		return internalCreateResolver(featureName2CollectInTokenResolver, featureName);
	}
	
	protected boolean registerTokenResolver(String tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver resolver){
		return internalRegisterTokenResolver(tokenName2TokenResolver, tokenName, resolver);
	}
	
	protected boolean registerCollectInTokenResolver(String featureName, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver resolver){
		return internalRegisterTokenResolver(featureName2CollectInTokenResolver, featureName, resolver);
	}
	
	protected org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver deRegisterTokenResolver(String tokenName){
		return tokenName2TokenResolver.remove(tokenName);
	}
	
	private org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver internalCreateResolver(java.util.Map<String, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver> resolverMap, String key) {
		if (resolverMap.containsKey(key)){
			return resolverMap.get(key);
		} else {
			return defaultResolver;
		}
	}
	
	private boolean internalRegisterTokenResolver(java.util.Map<String, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver> resolverMap, String key, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver resolver) {
		if (!resolverMap.containsKey(key)) {
			resolverMap.put(key,resolver);
			return true;
		}
		return false;
	}
	
}
