/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.templateconcepts.call.resource.templatecall.ui;

/**
 * An adapter from the Eclipse
 * <code>org.eclipse.jface.text.rules.ITokenScanner</code> interface to the
 * generated lexer.
 */
public class TemplatecallTokenScanner implements org.eclipse.jface.text.rules.ITokenScanner {
	
	private org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextScanner lexer;
	private org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextToken currentToken;
	private java.util.List<org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextToken> nextTokens;
	private int offset;
	private String languageId;
	private org.eclipse.jface.preference.IPreferenceStore store;
	private org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallColorManager colorManager;
	private org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextResource resource;
	
	/**
	 * 
	 * @param colorManager A manager to obtain color objects
	 */
	public TemplatecallTokenScanner(org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextResource resource, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallColorManager colorManager) {
		this.resource = resource;
		this.colorManager = colorManager;
		this.lexer = new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallMetaInformation().createLexer();
		this.languageId = new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallMetaInformation().getSyntaxName();
		org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallUIPlugin plugin = org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallUIPlugin.getDefault();
		if (plugin != null) {
			this.store = plugin.getPreferenceStore();
		}
		this.nextTokens = new java.util.ArrayList<org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextToken>();
	}
	
	public int getTokenLength() {
		return currentToken.getLength();
	}
	
	public int getTokenOffset() {
		return offset + currentToken.getOffset();
	}
	
	public org.eclipse.jface.text.rules.IToken nextToken() {
		boolean isOriginalToken = true;
		if (!nextTokens.isEmpty()) {
			currentToken = nextTokens.remove(0);
			isOriginalToken = false;
		} else {
			currentToken = lexer.getNextToken();
		}
		if (currentToken == null || !currentToken.canBeUsedForSyntaxHighlighting()) {
			return org.eclipse.jface.text.rules.Token.EOF;
		}
		
		if (isOriginalToken) {
			splitCurrentToken();
		}
		
		org.eclipse.jface.text.TextAttribute textAttribute = null;
		String tokenName = currentToken.getName();
		if (tokenName != null) {
			org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle staticStyle = getStaticTokenStyle();
			// now call dynamic token styler to allow to apply modifications to the static
			// style
			org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle dynamicStyle = getDynamicTokenStyle(staticStyle);
			if (dynamicStyle != null) {
				textAttribute = getTextAttribute(dynamicStyle);
			}
		}
		
		return new org.eclipse.jface.text.rules.Token(textAttribute);
	}
	
	public void setRange(org.eclipse.jface.text.IDocument document, int offset, int length) {
		this.offset = offset;
		try {
			lexer.setText(document.get(offset, length));
		} catch (org.eclipse.jface.text.BadLocationException e) {
			// ignore this error. It might occur during editing when locations are outdated
			// quickly.
		}
	}
	
	public String getTokenText() {
		return currentToken.getText();
	}
	
	public int[] convertToIntArray(org.eclipse.swt.graphics.RGB rgb) {
		if (rgb == null) {
			return null;
		}
		return new int[] {rgb.red, rgb.green, rgb.blue};
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle getStaticTokenStyle() {
		org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle staticStyle = null;
		String tokenName = currentToken.getName();
		String enableKey = org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.getPreferenceKey(languageId, tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.StyleProperty.ENABLE);
		boolean enabled = store.getBoolean(enableKey);
		if (enabled) {
			String colorKey = org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.getPreferenceKey(languageId, tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.StyleProperty.COLOR);
			org.eclipse.swt.graphics.RGB foregroundRGB = org.eclipse.jface.preference.PreferenceConverter.getColor(store, colorKey);
			org.eclipse.swt.graphics.RGB backgroundRGB = null;
			boolean bold = store.getBoolean(org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.getPreferenceKey(languageId, tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.StyleProperty.BOLD));
			boolean italic = store.getBoolean(org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.getPreferenceKey(languageId, tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.StyleProperty.ITALIC));
			boolean strikethrough = store.getBoolean(org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.getPreferenceKey(languageId, tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.StyleProperty.STRIKETHROUGH));
			boolean underline = store.getBoolean(org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.getPreferenceKey(languageId, tokenName, org.emftext.language.templateconcepts.call.resource.templatecall.ui.TemplatecallSyntaxColoringHelper.StyleProperty.UNDERLINE));
			staticStyle = new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTokenStyle(convertToIntArray(foregroundRGB), convertToIntArray(backgroundRGB), bold, italic, strikethrough, underline);
		}
		return staticStyle;
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle getDynamicTokenStyle(org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle staticStyle) {
		org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallDynamicTokenStyler dynamicTokenStyler = new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallDynamicTokenStyler();
		org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle dynamicStyle = dynamicTokenStyler.getDynamicTokenStyle(resource, currentToken, staticStyle);
		return dynamicStyle;
	}
	
	public org.eclipse.jface.text.TextAttribute getTextAttribute(org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenStyle tokeStyle) {
		int[] foregroundColorArray = tokeStyle.getColorAsRGB();
		org.eclipse.swt.graphics.Color foregroundColor = null;
		if (colorManager != null) {
			foregroundColor = colorManager.getColor(new org.eclipse.swt.graphics.RGB(foregroundColorArray[0], foregroundColorArray[1], foregroundColorArray[2]));
		}
		int[] backgroundColorArray = tokeStyle.getBackgroundColorAsRGB();
		org.eclipse.swt.graphics.Color backgroundColor = null;
		if (backgroundColorArray != null) {
			org.eclipse.swt.graphics.RGB backgroundRGB = new org.eclipse.swt.graphics.RGB(backgroundColorArray[0], backgroundColorArray[1], backgroundColorArray[2]);
			if (colorManager != null) {
				backgroundColor = colorManager.getColor(backgroundRGB);
			}
		}
		int style = org.eclipse.swt.SWT.NORMAL;
		if (tokeStyle.isBold()) {
			style = style | org.eclipse.swt.SWT.BOLD;
		}
		if (tokeStyle.isItalic()) {
			style = style | org.eclipse.swt.SWT.ITALIC;
		}
		if (tokeStyle.isStrikethrough()) {
			style = style | org.eclipse.jface.text.TextAttribute.STRIKETHROUGH;
		}
		if (tokeStyle.isUnderline()) {
			style = style | org.eclipse.jface.text.TextAttribute.UNDERLINE;
		}
		return new org.eclipse.jface.text.TextAttribute(foregroundColor, backgroundColor, style);
	}
	
	/**
	 * Tries to split the current token if it contains task items.
	 */
	public void splitCurrentToken() {
		final String text = currentToken.getText();
		final String name = currentToken.getName();
		final int line = currentToken.getLine();
		final int charStart = currentToken.getOffset();
		final int column = currentToken.getColumn();
		
		java.util.List<org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTaskItem> taskItems = new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTaskItemDetector().findTaskItems(text, line, charStart);
		
		// this is the offset for the next token to be added
		int offset = charStart;
		int itemBeginRelative;
		java.util.List<org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextToken> newItems = new java.util.ArrayList<org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextToken>();
		for (org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTaskItem taskItem : taskItems) {
			int itemBegin = taskItem.getCharStart();
			int itemLine = taskItem.getLine();
			int itemColumn = 0;
			
			itemBeginRelative = itemBegin - charStart;
			// create token before task item (TODO if required)
			String textBefore = text.substring(offset - charStart, itemBeginRelative);
			int textBeforeLength = textBefore.length();
			newItems.add(new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTextToken(name, textBefore, offset, textBeforeLength, line, column, true));
			
			// create token for the task item itself
			offset = offset + textBeforeLength;
			String itemText = taskItem.getKeyword();
			int itemTextLength = itemText.length();
			newItems.add(new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTextToken(org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTokenStyleInformationProvider.TASK_ITEM_TOKEN_NAME, itemText, offset, itemTextLength, itemLine, itemColumn, true));
			
			offset = offset + itemTextLength;
		}
		
		if (!taskItems.isEmpty()) {
			// create token after last task item (TODO if required)
			String textAfter = text.substring(offset - charStart);
			newItems.add(new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTextToken(name, textAfter, offset, textAfter.length(), line, column, true));
		}
		
		if (!newItems.isEmpty()) {
			// replace tokens
			currentToken = newItems.remove(0);
			nextTokens = newItems;
		}
		
	}
}
